import redis
from healthy_django.healthcheck.base import HealthCheck


class DjangoCeleryQueueLengthHealthCheck(HealthCheck):

    title = "Cache Check"

    required_params = ["broker", "queue_name", "info_length", "warning_length", "alert_length"]

    def check(self):
        try:
            redis_client = redis.from_url(self.params["broker"])
            queue_length = redis_client.llen(self.params["queue_name"])
            status = 200
            if queue_length < self.params["info_length"]:
                status = 200
            elif queue_length < self.params["warning_length"]:
                status = 300
            elif queue_length < self.params["alert_length"]:
                status = 400
            else:
                status = 500
            return status, {"queue_length": queue_length}
        except BaseException:
            return 500, {}


from healthy_django.healthcheck.django_cache import DjangoCacheHealthCheck
from django.conf import settings
from healthy_django.healthcheck.django_database import DjangoDatabaseHealthCheck
from healthy_django.healthcheck.celery_queue_length import DjangoCeleryQueueLengthHealthCheck
from healthy_django.healthcheck.sqs_length import AWSSQSQueueHealthCheck

default_configuration = []

HEALTH_CHECK = getattr(settings, "HEALTHY_DJANGO", default_configuration)
HEALTHY_DJANGO_AWS_REGION = getattr(settings, "HEALTHY_DJANGO_AWS_REGION", None)
HEALTHY_DJANGO_AWS_ACCESS_KEY = getattr(settings, "HEALTHY_DJANGO_AWS_ACCESS_KEY", None)
HEALTHY_DJANGO_AWS_SECRET = getattr(settings, "HEALTHY_DJANGO_AWS_SECRET", None)



from healthy_django.healthcheck.base import HealthCheck

from django.db import connections
from django.db.utils import OperationalError


class DjangoDatabaseHealthCheck(HealthCheck):

    title = "Database Check"

    required_params = ["connection_name"]

    def check(self):
        db_conn = connections[self.params["connection_name"]]
        try:
            c = db_conn.cursor()
            c.execute("select 1")
            connected = c.fetchone()[0] == 1
            if not connected:
                raise ConnectionError
            return 200, {}
        except OperationalError:
            return 500, {}
        except BaseException:
            return 503, {}


from healthy_django.healthcheck.base import HealthCheck

from django.core.cache import CacheKeyWarning, caches


class DjangoCacheHealthCheck(HealthCheck):

    title = "Cache Check"

    required_params = ["connection_name"]

    def check(self):
        cache = caches[self.params["connection_name"]]
        try:
            cache.set("healthy_django_test", 1)
            if not cache.get("healthy_django_test") == 1:
                raise ValueError("Cache returned wrong value")
            return 200, {}
        except BaseException:
            return 500, {}


# -*- coding: utf-8 -*-
import healthy_django
from healthy_django.views import HealthCheckView, HealthCheckInduvidualView, StatusPage
from django.urls import path
from django.views.generic import TemplateView
from healthy_django.conf import HEALTH_CHECK


app_name = "healthy_django"
urlpatterns = [
    path("statuspage", StatusPage.as_view()),
    path("", HealthCheckView.as_view()),
]

for check in HEALTH_CHECK:
    urlpatterns.append(
        path("{0}".format(check.slug), HealthCheckInduvidualView.as_view(), {"health": check}),
    )



import boto3
from healthy_django.healthcheck.base import HealthCheck


class AWSSQSQueueHealthCheck(HealthCheck):

    title = "SQS Queue Length Check"

    required_params = ["info_length", "warning_length", "alert_length"]

    def check(self):
        from healthy_django.conf import (
            HEALTHY_DJANGO_AWS_ACCESS_KEY,
            HEALTHY_DJANGO_AWS_REGION,
            HEALTHY_DJANGO_AWS_SECRET,
        )

        try:
            client_args = {
                "region_name": HEALTHY_DJANGO_AWS_REGION,
                "aws_access_key_id": HEALTHY_DJANGO_AWS_ACCESS_KEY,
                "aws_secret_access_key": HEALTHY_DJANGO_AWS_SECRET,
            }
            sqs_client = boto3.resource("sqs", **client_args)
            queue_url = None
            if "queue_url" in self.params:
                queue_url = self.params["queue_url"]
            elif "queue_name" in self.params:
                queue_url = sqs_client.get_queue_url(
                    QueueName=self.params["queue_name"],
                )["QueueUrl"]
            queue = sqs_client.Queue(self.params["queue_url"])
            queue_length = int(queue.attributes["ApproximateNumberOfMessages"])
            status = 200
            if queue_length < self.params["info_length"]:
                status = 200
            elif queue_length < self.params["warning_length"]:
                status = 300
            elif queue_length < self.params["alert_length"]:
                status = 400
            else:
                status = 500
            return status, {"queue_length": queue_length}
        except BaseException as e:
            print(e)
            return 500, {}


import time
from healthy_django.exceptions import InvalidParams


class HealthCheck:

    name = ""
    title = ""
    description = ""
    slug = ""
    exclude_main = False
    health_code_pretty = {200: "All OK", 500: "Down"}

    required_params = []
    params = {}

    def __init__(self, name, slug, exclude_main=False, **params):
        self.name = name
        self.slug = slug
        self.exclude_main = exclude_main
        for param in self.required_params:
            if param not in params:
                raise InvalidParams("Param {0} is required for checking health".format(param))
        self.params = params

    def check(self):
        raise NotImplemented()

    def health_status(self):
        cur_time = time.time()
        health_code, health_meta = self.check()
        latency = time.time() - cur_time
        return {
            "name": self.name,
            "title": self.title,
            "code": health_code,
            "message": self.health_code_pretty.get(health_code, ""),
            "latency": float("{0:.4f}".format(latency)),
            "meta": health_meta,
        }



from django.http.response import HttpResponse, JsonResponse
from django.shortcuts import render
from django.utils.decorators import method_decorator
from django.views import View
from django.views.decorators.csrf import csrf_exempt

from healthy_django.conf import HEALTH_CHECK


def get_status():
    results = []
    status_code = 200
    for check in HEALTH_CHECK:
        if check.exclude_main:
            continue
        status = check.health_status()
        results.append(status)
        status_code = max(status["code"], status_code)
    return results, status_code


class HealthCheckView(View):
    def get(self, request):
        results, status_code = get_status()
        return JsonResponse({"health": results}, status=status_code)


class HealthCheckInduvidualView(View):
    def get(self, request, health):
        status = health.health_status()
        status_code = status["code"]
        return JsonResponse({"health": status}, status=status_code)


class StatusPage(View):
    @method_decorator(csrf_exempt)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)

    def get(self, request):
        return render(request, "healthy_django/index.html")

    def post(self, request):
        results, status_code = get_status()
        return JsonResponse({"health": results}, status=status_code)
